<?php

namespace Drupal\acquia_cloud_backup_manager;

use AcquiaCloudApi\Endpoints\Databases;
use Drupal\Core\Config\ImmutableConfig;
use AcquiaCloudApi\Connector\Client;
use AcquiaCloudApi\Connector\Connector;
use AcquiaCloudApi\Endpoints\Applications;
use AcquiaCloudApi\Endpoints\DatabaseBackups;
use AcquiaCloudApi\Endpoints\Environments;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Acquia Cloud Client.
 */
class AcquiaCloudClient {

  /**
   * API Secret environment var.
   */
  const API_SECRET_VAR_NAME = 'CLOUD_PLATFORM_API_SECRET';

  /**
   * API Token environment var.
   */
  const API_TOKEN_VAR_NAME = 'CLOUD_PLATFORM_API_TOKEN';

  /**
   * Acquia Cloud client.
   *
   * @var \AcquiaCloudApi\Connector\Client|null
   */
  private ?Client $client = NULL;

  /**
   * Acquia cloud settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private ImmutableConfig $config;

  /**
   * Load dependencies.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->config = $config_factory->get('acquia_cloud_backup_manager.settings');

    if ($this->environmentVariablesAvailables()) {
      /** @var string $token */
      $token = getenv(self::API_TOKEN_VAR_NAME);
      /** @var string $secret */
      $secret = getenv(self::API_SECRET_VAR_NAME);

      $this->setClient($token, $secret);
    }
    else {
      $this->client = $this->getClient();
    }

  }

  /**
   * Get Acquia Cloud client.
   */
  public function getClient() {
    if (!($this->client instanceof Client)) {
      if (!(getenv(self::API_TOKEN_VAR_NAME) && getenv(self::API_SECRET_VAR_NAME))) {
        $key = $this->config->get('key');
        $secret = $this->config->get('secret');
      }
      else {
        $key = self::API_TOKEN_VAR_NAME;
        $secret = self::API_SECRET_VAR_NAME;
      }

      $config = [
        'key' => $key,
        'secret' => $secret,
      ];

      $connector = new Connector($config);
      $client = Client::factory($connector);
      $this->client = $client;
    }

    return $this->client;
  }

  /**
   * Set Acquia Cloud Client.
   */
  public function setClient(string $key = '', string $secret = '') {
    $config = [
      'key' => $key,
      'secret' => $secret,
    ];

    $connector = new Connector($config);
    $client = Client::factory($connector);
    $this->client = $client;
  }

  /**
   * Get backup list.
   */
  public function getBackupList() {
    $client = $this->getClient();
    $backup = new DatabaseBackups($client);
    $environment_uuid = $this->config->get('environment_uuid');
    $database_name = $this->config->get('database_name');
    $backups = $backup->getAll($environment_uuid, $database_name);
    $backup_list = [];

    /** @var \AcquiaCloudApi\Response\BackupResponse $backup_item */
    foreach ($backups as $backup_item) {
      // Only treat the ondemand backups.
      if ($backup_item->type == 'ondemand') {
        $backup_list[$backup_item->id] = strtotime($backup_item->completedAt);
      }
    }
    return $backup_list;
  }

  /**
   * Get Acquia Cloud application list.
   */
  public function getApplicationList() {
    $client = $this->getClient();
    $application = new Applications($client);
    $applications_list = $application->getAll();
    $applications = [];

    /** @var \AcquiaCloudApi\Response\ApplicationResponse $application */
    foreach ($applications_list as $application) {
      $applications[$application->uuid] = $application->name;
    }
    return $applications;
  }

  /**
   * Get Acquia Cloud environment list.
   */
  public function getEnvironmentList(string $application_id) {
    $client = $this->getClient();
    $environments = new Environments($client);
    $environments_list = $environments->getAll($application_id);
    $environments = [];
    /** @var \AcquiaCloudApi\Response\EnvironmentResponse $environment */
    foreach ($environments_list as $environment) {
      $environments[$environment->uuid] = $environment->name;
    }
    return $environments;
  }

  /**
   * Ge the database list by the application id.
   */
  public function getDatabasesList(string $application_id) {
    $client = $this->getClient();
    $databases = new Databases($client);
    $databases_list = $databases->getAll($application_id);
    $databases = [];
    /** @var \AcquiaCloudApi\Response\DatabaseResponse $database */
    foreach ($databases_list as $database) {
      $databases[$database->name] = $database->name;
    }
    return $databases;
  }

  /**
   * Delete older backups that the time gived.
   *
   * @param int $time_to_keep
   *   The number of days of the backups to keep.
   */
  public function deleteOlderBackups(int $time_to_keep) {
    $backup_list = $this->getBackupList();
    if (empty($backup_list)) {
      return [];
    }

    $time_limit = strtotime('-' . $time_to_keep . 'days');
    $backups_to_delete = [];

    foreach ($backup_list as $uuid => $backup_time) {
      if ($backup_time < $time_limit) {
        $backups_to_delete[$uuid]['date'] = date(\DateTimeInterface::ATOM, $backup_time);
      }
    }

    $deleted_backups = [];
    if (!empty($backups_to_delete)) {
      $deleted_backups = $this->deleteBackups($backups_to_delete);
    }
    return $deleted_backups;
  }

  /**
   * Delete a list of backups by its uuid.
   */
  public function deleteBackups(array $backups_to_delete) {
    if (empty($backups_to_delete)) {
      return [];
    }
    $client = $this->getClient();
    $backups_client = new DatabaseBackups($client);
    $environment_uuid = $this->config->get('environment_uuid');
    $database_name = $this->config->get('database_name');

    foreach (array_keys($backups_to_delete) as $uuid) {
      $backups_client->delete($environment_uuid, $database_name, $uuid);
    }
    return $backups_to_delete;
  }

  /**
   * Check if API Credentials environment variables are available.
   */
  public function environmentVariablesAvailables() {
    return (getenv(self::API_TOKEN_VAR_NAME) && getenv(self::API_SECRET_VAR_NAME));
  }

}
